## RISC-V 基础整数指令集

### RV32I 指令格式（六种基本指令格式

​        用于寄存器-寄存器操作的 R 类型指令，用 于短立即数和访存 load 操作的 I 型指令，用于访存 store 操作的 S 型指令，用于条件跳转操 作的 B 类型指令，用于长立即数的 U 型指令和用于无条件跳转的 J 型指令。

![image-20210718165717213](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210718165717213.png)

![image-20210718164611401](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210718164611401.png)

![image-20210718165650188](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210718165650188.png)

**补充说明**：B 类型和 J 类型指令 如下所述，分支指令（B 类型）的立即数字段在 S 类型的基础上旋转了 1 位。跳转指令（J 类型）的直接字段在 U 类型的基础上旋转了 12 位。因此,RISC-V 实际上只有四种基本格式， 但我们可以保守地认为它有六种格式。

### RV32I 整数计算

​        简单的算术指令（add, sub）、逻辑指令（and, or, xor），以及图 2.1 中的移位指令 （sll, srl, sra）和其他 ISA 差不多。他们从寄存器读取两个 32 位的值，并将 32 位结果写 入目标寄存器。RV32I 还提供了这些指令的立即数版本。和 ARM-32 不同，立即数总是进 行符号扩展，这样子如果需要，我们可以用立即数表示负数，正因为如此，我们并不需要 一个立即数版本的 sub。

​        程序可以根据比较结果生成布尔值。为应对这种使用场景下，RV32I 提供一个当小于 时置位的指令。如果第一个操作数小于第二个操作数，它将目标寄存器设置为 1，否则为 0。不出所料，对这个指令，有一个有符号版本  （slt）和无符号版本（sltu），分别用于处 理有符号和无符号整数比较。相应的，上述两条指令也有立即数版本的（slti，sltiu）。正 如我们将要看到的，虽然 RV32I 分支指令可以检查两个寄存器之间的所有关系，但一些条 件表达式涉及多对寄存器之间的关系。对于这些表达式，编译器或汇编语言程序员可以将 slt 以及与或异或等逻辑指令组合使用来解决更复杂的条件表达式。 

​         剩下的两条整数计算指令主要用于构造大的常量数值和链接。加载立即数到高 位（lui）将 20 位常量加载到寄存器的高 20 位。接着便可以使用标准的立即指令来创建 32 位常量。这样子，仅使用 2 条 32 位 RV32I 指令，便可构造一个 32 位常量。向 PC 高位加 上立即数（auipc）让我们仅用两条指令，便可以基于当前 PC 以任意偏移量转移控制流或 者访问数据。将 auipc 中的 20 位立即数与 jalr（参见下面）中 12 位立即数的组合，我们 可以将执行流转移到任何 32 位 PC 相对地址。而 auipc 加上普通加载或存储指令中的 12 位立即数偏移量，使我们可以访问任何 32 位 PC 相对地址的数据。

### RV32I特性

⚫ 32 位字节可寻址的地址空间 

⚫ 所有指令均为 32 位长 

⚫ 31 个寄存器，全部 32 位宽，寄存器 0 硬连线为零 

⚫ 所有操作都在寄存器之间（没有寄存器到内存的操作） 

⚫ 加载/存储字加上有符号和无符号加载/存储字节和半字 

⚫ 所有算术，逻辑和移位指令都有立即数版本的指令 

⚫ 立即数总是符号扩展 

⚫ 仅提供一种数据寻址模式（寄存器+立即数）和 PC 相对分支 

⚫ 无乘法或除法指令 

⚫ 一个指令，用于将大立即数加载到寄存器的高位，这样加载 32 位常量到寄存器只 需要两条指令

## RISC-V 汇编语言

​        图 3.1 表明了从 C 程序翻译成为可以在计算机上执行的机器语言程序的四个经典步骤。

![image-20210718205555367](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210718205555367.png)

### 动态链接

​        现在的许多系统使用动态链接（dynamic linking），外部的函数在 第一次被调用时才会加载和链接。后续所有调用都使用快速链接（fast linking），因此只会产 生一次动态开销。每次程序开始运行，它都会按照需要链接最新版本的库函数。另外，如果 多个程序使用了同一个动态链接库，库代码在内存中只会加载一次。 

​        编译器产生的代码和静态链接的代码很相似。其不同之处在于，跳转的目标不是实际的 函数，而是一个只有三条指令的存根函数（stub function）。存根函数会从内存中的一个表中 加载实际的函数的地址并跳转。不过，在第一次调用时，表中还没有实际的函数的地址，只 有一个动态链接的过程的地址。当这个动态链接过程被调用时，动态链接器通过符号表找到 实际要调用的函数，复制到内存中，更新记录实际的函数地址的表。后续的每次调用的开销 就是存根函数的三条指令的开销。

### 加载器 

​        运行时，加载 器的作用是把这个程序加载到内存中，并跳转到它开始的地址。如今的“加载器”就是操作 系统。换句话说，加载 a.out 是操作系统众多的任务之一。 动态链接程序的加载稍微有些复杂。操作系统不直接运行程序，而是运行一个动态链接 器，再由动态链接器开始运行程序，并负责处理所有外部函数的第一次调用，把它们加载到 内存中，并且修改程序，填入正确的调用地址。

## 乘法和除法指令

![image-20210718214134907](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210718214134907.png)

![image-20210718214150803](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210718214150803.png)

## RV32F 和 RV32D：单精度和双精度浮点数

![image-20210718214400097](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210718214400097.png)

​        IEEE 754-2008 浮点标准[IEEE Standards Committee 2008]定义了浮点数据类型，计算精 度和所需操作。它的广泛流行大大降低了移植浮点程序的难度，这也意味着不同 ISA 中的浮 点数部分可能比其他章节中描述的其他部分的指令更一致。

## 原子指令

RV32A 有两种类型的原子操作：

 ⚫ 内存原子操作（AMO）

 ⚫ 加载保留/条件存储（load reserved / store conditional）

![image-20210718221010531](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210718221010531.png)

![image-20210718221217838](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210718221217838.png)

## 压缩指令

![image-20210718221803270](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210718221803270.png)

## 向量

最著名的数据级并行架构是单指令多数据(SIMD，Single Instruction Multiple Data)。 SIMD 最初的流行是因为它将 64 位寄存器的数据分成许多个 8 位、16 位或 32 位的部分， 然后并行地计算它们。操作码提供了数据宽度和操作类型。数据传输只用单个（宽）SIMD 寄存器的 load 和 store 进行。

一个更老的，并且在我们看来更优雅的，利用数据级并行性的方案是向量架构。将向量的长度和每个时钟周期可以进行的最大操作数分 离，是向量体系结构的关键所在：向量微架构可以灵活地设计数据并行硬件而不会影响到 程序员，程序员可以不用重写代码就享受到长向量带来的好处。

![image-20210721161737464](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210721161737464.png)



### 向量寄存器和动态类型

RV32V 添加了 32 个向量寄存器，它们的名称以 v 开头，但每个向量寄存器的元素个 数不同。该数量取决于操作的宽度和专用于向量寄存器的存储大小，而这取决于处理器的 设计者。比方说，如果处理器为向量寄存器分配了 4096 个字节，则这足以让这些 32 个向 量寄存器中有 16 个 64 位元素，或者 32 个 32 位元素，或者 64 个 16 位元素，或 128 个 8 位元素。 为了在向量 ISA 中保持元素数量的灵活性，向量处理器会计算会最大向量长度 （mvl），即在给定的容量限制下，向量程序使用这个向量寄存器可以运算的最大向量长 度。向量长度寄存器（vl）为特定操作设定了向量中含有的元素数量，这有助于数组维度 不是 mvl 的整数倍时的编程。

**补充说明：**RV32V 可以快速切换上下文 向量架构不如 SIMD 架构受欢迎的一个原因是：大家担心增加大型向量寄存器会延长中断 时保存和恢复程序（上下文切换）的时间。动态寄存器类型对此很有帮助。程序员必须告诉 处理器正在使用哪些向量寄存器，这意味着处理器需要在上下文切换中仅保存和恢复那些 寄存器。

## RV64：64 位地址指令

要切换 到 64 位 ISA，ISA 只添加了少数指令。指令集只添加了 32 位指令对应的字(word)，双字 (doubleword)和长整数(long)版本的指令，并将所有寄存器（包括 PC）扩展为 64 位。因此， RV64I 中的 sub 操作的是两个 64 位数字而不是 RV32I 中的 32 位数字。RV64 很接近 RV32 但实际上又有所不同;它添加了少量指令同时基础指令做的事情与 RV32 中稍有不同。

![image-20210721164350689](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210721164350689.png)

![image-20210721164435962](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210721164435962.png)

![image-20210721164447253](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210721164447253.png)

![image-20210721164500069](C:\Users\Leedia\AppData\Roaming\Typora\typora-user-images\image-20210721164500069.png)