### 实验题

1. 原理：在 `rust_main` 函数中，执行 `ebreak` 命令后至函数结束前，`sp` 寄存器的值是怎样变化的？

   在rust_main函数中执行`ebreak`进入中断，首先需要在栈上开辟Context所需空间，即将栈顶减去 34*8 bits(数组/结构体寻址则是基地址加上偏移量，所以sp可以作为context的地址，	在`handle_interrupt`里调用了`breakpoint`函数，这时有新的参数入栈和出栈。最后在`restore`中恢复`sp`原来的值。。

2. 分析：如果去掉 `rust_main` 后的 `panic` 会发生什么，为什么？

   去掉rust_main后的`panic`后程序将不会立刻返回。会继续执行后面的任意代码段，很容易出现非法访问。

3. 实验

   1. 如果程序访问不存在的地址，会得到 `Exception::LoadFault`。模仿捕获 `ebreak` 和时钟中断的方法，捕获 `LoadFault`（之后 `panic` 即可）。

          如下代码：

      ```
      pub fn handle_interrupt(context: &mut Context, scause: Scause, stval: usize) {
          // 返回的 Context 必须位于放在内核栈顶
          match scause.cause() {
              // 断点中断（ebreak）
              Trap::Exception(Exception::Breakpoint) => breakpoint(context),
              //loadFault
              Trap::Exception(Exception::LoadFault)=>loadfault(context, stval),
              // 时钟中断
              Trap::Interrupt(Interrupt::SupervisorTimer) => supervisor_timer(context),
              
              // 其他情况，终止当前线程
              _ => fault(context, scause, stval),
          };
      ```

      

   2. 在处理异常的过程中，如果程序想要非法访问的地址是 `0x0`，则打印 `SUCCESS!`。

      ```rust
      //handle_interrupt已经获取了stval并且把它当成参数传入了，因此直接判断即可
      fn load_falut(context: &mut Context, scause: Scause, stval: usize) {
          if (stval == 0x0){
              println!("Success!")
          }
          panic!(
              "Interrupt: {:?}\n{:x?}\nstval: {:x}",
              scause.cause(),
              context,
              stval
          )
      }
      ```

      

   3. 添加或修改少量代码，使得运行时触发这个异常，并且打印出 `SUCCESS!`。

      可以直接修改达到则 `sret` 时程序会跳转到 `0x0`的目的，或者最后跳转到0x0