### 实验题

1. 原理：.bss 字段是什么含义？为什么我们要将动态分配的内存（堆）空间放在 .bss 字段？

   `.bss`段：未初始化的全局和静态变量，以及所有被初始化为0的全局或静态变量，在目标文件中这个这个节不占据实际的空间，它仅仅是一个占位符。目标文件格式区分已初始化和未初始化变量是为了空间效率：在目标文件中，未初始化变量不需要占据任何实际的磁盘空间。运行时，在内存中分配这些变量，初始化值为0。

   放在`.bss`段，实际并不是必须的，我们完全可以随意指定一段可以访问的内存空间。但是放在.bss段实现很简单。

   

2. 分析：我们在动态内存分配中实现了一个堆，它允许我们在内核代码中使用动态分配的内存，例如 `Vec` `Box` 等。那么，如果我们在实现这个堆的过程中使用 `Vec` 而不是 `[u8]`，会出现什么结果？

   - 无法编译？
   - 运行时错误？
   - 正常运行？

   都不会，会发生死锁。使用`Vec`会向堆中申请内存，而`Allocator`也需要向堆内申请内存。

3. 实验

   1. 回答：`algorithm/src/allocator` 下有一个 `Allocator` trait，我们之前用它实现了物理页面分配。这个算法的时间和空间复杂度是什么？

      ```rust
      /// 分配器：固定容量，每次分配 / 回收一个元素 
      pub trait Allocator {    
      /// 给定容量，创建分配器    
      fn new(capacity: usize) -> Self;    
      /// 分配一个元素，无法分配则返回 `None`    
      fn alloc(&mut self) -> Option<usize>;    
      /// 回收一个元素    
      fn dealloc(&mut self, index: usize); 
      }
      ```

      时间复杂度是 O(1)，空间复杂度是 O(n)

   2. 二选一：实现基于线段树的物理页面分配算法（不需要考虑合并分配）；或尝试修改 `FrameAllocator`，令其使用未被分配的页面空间（而不是全局变量）来存放页面使用状态。

   具体代码见：algorithm/src/allocator/stacked_allocator.rs

