### 实验题

1. 原理：在 `os/src/entry.asm` 中，`boot_page_table` 的意义是什么？当跳转执行 `rust_main` 时，不考虑缓存，硬件通过哪些地址找到了 `rust_main` 的第一条指令？

   结合答案和查询资料得出下面详细解释：

   ```text
   _start:
       # 通过线性映射关系计算 boot_page_table 的物理页号
       lui t0, %hi(boot_page_table)
       li t1, 0xffffffff00000000
       sub t0, t0, t1
       srli t0, t0, 12
       # 8 << 60 是 satp 中使用 Sv39 模式的记号
       li t1, (8 << 60)
       or t0, t0, t1
       # 写入 satp 并更新 TLB
       csrw satp, t0
       sfence.vma
   ```

   首先声明代码段位置并且实现在`linker.ld`中声明的`__start`函数，随后我们做的就是把`boot_page_table`（即页表基址）的虚拟地址转化成物理地址写入`satp`寄存器中，我们来看是如何把`boot_page_table`的物理页号写入的：

   由于在`rCore`中，我们选择了RISC -V本身硬件支持Sv39模式作为页表的实现。

   在 Sv39 模式中，定义物理地址有 56 位，而虚拟地址有 64 位。虽然虚拟地址有 64 位，只有低 39 位有效。不过这不是说高 25 位可以随意取值，规定 63-39 位的值必须等于第 38 位的值，否则会认为该虚拟地址不合法，在访问时会产生异常。

   Sv39 模式同样是基于页的，在物理内存那一节曾经提到**物理页（Frame）**与**物理页号（PPN，Physical Page Number）**。在这里物理页号为 44 位，每个物理页大小为 4KB。同理，我们对于虚拟内存定义**虚拟页（Page）**以及**虚拟页号（VPN, Virtual Page Number)** 。在这里虚拟页号为 27 位，每个虚拟页大小也为 4KB。物理地址和虚拟地址的最后 12 位都表示页内偏移，即表示该地址在所在物理页（虚拟页）上的什么位置。

   首先，我们取`boot_page_table`地址的12位到32位，此为物理地址的页号，然后将物理地址页号与`0xffffffff00000000`进行相减存到`t0`寄存器中，然后将`t0`寄存器右移12位，然后我们将`(8 << 60)`作为Sv39模式的记号存入到`t1`寄存器中，然后将`t0`和`t1`寄存器进行或运算存入到`satp`寄存器中，并且更新快表。这样我们就可以通过`satp`寄存器的值来找到页表基址。

   随后我们计算栈起始地址并将其写入`sp`寄存器中，并且计算出`rust_main`函数的虚拟地址并写入临时寄存器中。此时我们已经有了`rust_main`的虚拟地址了，我们可以从虚拟地址（即高地址）首先取其高九位（即VPN3）判断其寻找VPN2的地址（此时应为510），然后再进行不断的映射最后经过`satp`寄存器找到`rust_main`的物理地址进行运行。

2. 分析：为什么 `Mapping` 中的 `page_tables` 和 `mapped_pairs` 都保存了一些 `FrameTracker`？二者有何不同？

   `page_tables`存放的是所有页表所使用的页面，而`mapped_pairs`则存放着进程所需要的页面。

   

3. 分析：假设某进程需要虚拟地址 A 到物理地址 B 的映射，这需要操作系统来完成。那么操作系统在建立映射时有没有访问 B？如果有，它是怎么在还没有映射的情况下访问 B 的呢？

   建立映射不需要访问 B，而只需要操作页表即可。

   

